@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,14 +20,34 @@
 #include "config.h"
 
 #define _XOPEN_SOURCE
+
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include <time.h>
 #include <string.h>
+#include <stdio.h>
 
 #include <libtracker-common/tracker-utils.h>
 #include <libtracker-common/tracker-date-time.h>
 
 #include "tracker-utils.h"
 
+#ifndef HAVE_GETLINE
+
+#include <stddef.h>
+#include <stdlib.h>
+#include <limits.h>
+#include <errno.h>
+
+#undef getdelim
+#undef getline
+
+#define GROW_BY 80
+
+#endif /* HAVE_GETLINE */
+
 #define DATE_FORMAT_ISO8601 "%Y-%m-%dT%H:%M:%S%z"
 
 /**
@@ -53,6 +73,49 @@ static const char imonths[] = {
 	'6', '7', '8', '9', '0', '1', '2'
 };
 
+
+/**
+ * tracker_coalesce_strip:
+ * @n_values: the number of @Varargs supplied
+ * @Varargs: the string pointers to coalesce
+ *
+ * This function iterates through a series of string pointers passed
+ * using @Varargs and returns the first which is not %NULL, not empty
+ * (i.e. "") and not comprised of one or more spaces (i.e. " ").
+ *
+ * The returned value is stripped using g_strstrip(). It is MOST
+ * important NOT to pass constant string pointers to this function!
+ *
+ * Returns: the first string pointer from those provided which
+ * matches, otherwise %NULL.
+ *
+ * Since: 0.9
+ **/
+const gchar *
+tracker_coalesce_strip (gint n_values,
+                        ...)
+{
+	va_list args;
+	gint    i;
+	const gchar *result = NULL;
+
+	va_start (args, n_values);
+
+	for (i = 0; i < n_values; i++) {
+		gchar *value;
+
+		value = va_arg (args, gchar *);
+		if (!result && !tracker_is_blank_string (value)) {
+			result = (const gchar *) g_strstrip (value);
+			break;
+		}
+	}
+
+	va_end (args);
+
+	return result;
+}
+
 /**
  * tracker_coalesce:
  * @n_values: the number of @Varargs supplied
@@ -70,6 +133,9 @@ static const char imonths[] = {
  * matches, otherwise %NULL.
  *
  * Since: 0.8
+ *
+ * Deprecated: 1.0: Use tracker_coalesce_strip() instead.
+ *
  **/
 gchar *
 tracker_coalesce (gint n_values,
@@ -97,8 +163,9 @@ tracker_coalesce (gint n_values,
 	return result;
 }
 
+
 /**
- * tracker_merge:
+ * tracker_merge_const:
  * @delimiter: the delimiter to use when merging
  * @n_values: the number of @Varargs supplied
  * @Varargs: the string pointers to merge
@@ -113,7 +180,63 @@ tracker_coalesce (gint n_values,
  * Returns: a newly-allocated string holding the result which should
  * be freed with g_free() when finished with, otherwise %NULL.
  *
+ * Since: 0.9
+ **/
+gchar *
+tracker_merge_const (const gchar *delimiter, 
+                     gint         n_values,
+                     ...)
+{
+	va_list args;
+	gint    i;
+	GString *str = NULL;
+
+	va_start (args, n_values);
+
+	for (i = 0; i < n_values; i++) {
+		gchar *value;
+
+		value = va_arg (args, gchar *);
+		if (value) {
+			if (!str) {
+				str = g_string_new (value);
+			} else {
+				if (delimiter) {
+					g_string_append (str, delimiter);
+				}
+				g_string_append (str, value);
+			}
+		}
+	}
+
+	va_end (args);
+
+	if (!str) {
+		return NULL;
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+/**
+ * tracker_merge:
+ * @delimiter: the delimiter to use when merging
+ * @n_values: the number of @Varargs supplied
+ * @Varargs: the string pointers to merge
+ *
+ * This function iterates through a series of string pointers passed
+ * using @Varargs and returns a newly allocated string of the merged
+ * strings. All passed strings are freed (don't pass const values)/
+ *
+ * The @delimiter can be %NULL. If specified, it will be used in
+ * between each merged string in the result.
+ *
+ * Returns: a newly-allocated string holding the result which should
+ * be freed with g_free() when finished with, otherwise %NULL.
+ *
  * Since: 0.8
+ *
+ * Deprecated: 1.0: Use tracker_merge_const() instead.
  **/
 gchar *
 tracker_merge (const gchar *delimiter, 
@@ -649,3 +772,91 @@ tracker_date_guess (const gchar *date_string)
 
 	return g_strdup (date_string);
 }
+
+/**
+ * tracker_getline:
+ * @linebuf: Buffer to write into
+ * @n: Max bytes of linebuf
+ * @stream: Filestream to read from
+ *
+ * Reads an entire line from stream, storing the address of the buffer
+ * containing  the  text into *lineptr.  The buffer is null-terminated
+ * and includes the newline character, if one was found.
+ *
+ * Read GNU getline()'s manpage for more information
+ *
+ * Since: 0.9
+ **/
+
+#ifndef HAVE_GETLINE
+static gint
+my_igetdelim (gchar  **linebuf,
+              guint   *linebufsz,
+              gint     delimiter,
+              FILE    *file)
+{
+	gint ch;
+	gint idx;
+
+	if ((file == NULL || linebuf == NULL || *linebuf == NULL || *linebufsz == 0) &&
+	    !(*linebuf == NULL && *linebufsz == 0)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (*linebuf == NULL && *linebufsz == 0) {
+		*linebuf = g_malloc (GROW_BY);
+
+		if (!*linebuf) {
+			errno = ENOMEM;
+			return -1;
+		}
+
+		*linebufsz += GROW_BY;
+	}
+
+	idx = 0;
+
+	while ((ch = fgetc (file)) != EOF) {
+		/* Grow the line buffer as necessary */
+		while (idx > *linebufsz - 2) {
+			*linebuf = g_realloc (*linebuf, *linebufsz += GROW_BY);
+
+			if (!*linebuf) {
+				errno = ENOMEM;
+				return -1;
+			}
+		}
+		(*linebuf)[idx++] = (gchar) ch;
+
+		if ((gchar) ch == delimiter) {
+			break;
+		}
+	}
+
+	if (idx != 0) {
+		(*linebuf)[idx] = 0;
+	} else if ( ch == EOF ) {
+		return -1;
+	}
+
+	return idx;
+}
+
+gssize
+tracker_getline (gchar **lineptr,
+                 gsize  *n,
+                 FILE *stream)
+{
+	return my_igetdelim (lineptr, n, '\n', stream);
+}
+
+#else
+gssize
+tracker_getline (gchar **lineptr,
+                 gsize  *n,
+                 FILE *stream)
+{
+	return getline (lineptr, n, stream);
+}
+#endif /* HAVE_GETLINE */
